﻿using System;
using MVR.FileManagementSecure;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace ky1001
{
    public class CUAThumbnailHelper : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "ky1001";
        public string pluginName = "CUAThumbnailHelper";
        public string pluginVersion = "3.0";
        public string pluginDate = "2022-11-14";
        public string pluginDescription = "";
        #endregion

        private const string _cuaPath = "Custom/Assets/";
        private const string _cuaExt = "assetbundle";
        private const string _addonPackagesPath = "AddonPackages";
        private const string _filePrefs = "AddonPackagesFilePrefs/";
        private const string _pluginDataPath = "Saves/PluginData/ky1001/CUAThumbnailHelper/";
        private const int _jsonstringLimit = 15000;
        private Atom _targetAtom;
        private JSONStorable _cuaControl;
        private JSONStorableString _cuaListTF;
        private JSONStorableString _currentCUATF;
        private JSONStorableString _assetListTF;
        private JSONStorableStringChooser _selectAsset;
        private int _curretCUANumber;
        private int _maxCUANumber;
        private string _cuaListText;
        private string _currentCUAText;
        private string _assetListText;
        private string _selectedPath;
        private int _startTime;
        private int _startTime2;
        private bool _isNeedAssetRefresh;
        private bool _isWaitRefreshList;
        private JSONStorableBool _isEnableVar;
        private UIDynamicToggle _enableVarToggle;
        private JSONStorableFloat _cuaScale;
        private JSONStorableStringChooser _lightSelect;
        private JSONStorableFloat _intensity;
        private Atom _invisibleLight;

        private List<string> _cuaList = new List<string>();
        private List<string> _assetList = new List<string>();

        public override void Init()
        {
            try
            {
                _targetAtom = containingAtom;
                _cuaControl = _targetAtom.GetStorableByID("asset");

                _isEnableVar = new JSONStorableBool("Enable VAR", false);
                var _enableVarToggle = CreateToggle(_isEnableVar);

                var selectDirButton = CreateButton("Select Assetbundle Directory");
                selectDirButton.button.onClick.AddListener(SelectDir);
                JSONStorableAction SelectDirAct = new JSONStorableAction("Select Assetbundle Directory", SelectDir);

                var screenshotButton = CreateButton("Screenshot", true);
                screenshotButton.button.onClick.AddListener(Screenshot);
                JSONStorableAction ScreenshotAct = new JSONStorableAction("Screenshot", Screenshot);
                screenshotButton.buttonColor = Color.green;

                _cuaListText = "";
                _cuaListTF = new JSONStorableString("", _cuaListText);
                var textBox = CreateTextField(_cuaListTF);
                textBox.height = 400;

                var nextCUAButton = CreateButton("Assetbundle List >");
                nextCUAButton.buttonColor = Color.yellow;
                nextCUAButton.button.onClick.AddListener(NextCUA);
                JSONStorableAction NextCUAAct = new JSONStorableAction("Assetbundle List >", NextCUA);

                var nextCUAButton2 = CreateButton("Assetbundle List +10 >>");
                nextCUAButton2.button.onClick.AddListener(NextCUA2);
                JSONStorableAction NextCUAAct2 = new JSONStorableAction("Assetbundle List +10 >>", NextCUA2);

                var previousCUAButton = CreateButton("< Assetbundle List");
                previousCUAButton.button.onClick.AddListener(PreviousCUA);
                JSONStorableAction PreviousCUAAct = new JSONStorableAction("< Assetbundle List", PreviousCUA);

                var previousCUAButton2 = CreateButton("<< Assetbundle List -10");
                previousCUAButton2.button.onClick.AddListener(PreviousCUA2);
                JSONStorableAction PreviousCUAAct2 = new JSONStorableAction("<< Assetbundle List -10", PreviousCUA2);

                _currentCUAText = "";
                _currentCUATF = new JSONStorableString("", _currentCUAText);
                CreateTextField(_currentCUATF);

                var selectCUAButton = CreateButton("Select Assetbundle");
                selectCUAButton.buttonColor = Color.cyan;
                selectCUAButton.button.onClick.AddListener(SelectCUA);
                JSONStorableAction selectCUAAct = new JSONStorableAction("Select Assetbundle", SelectCUA);

                _assetListText = "";
                _assetListTF = new JSONStorableString("", _assetListText);
                CreateTextField(_assetListTF, true);

                _selectAsset = new JSONStorableStringChooser("Select Asset", null, "None", "Select Asset");
                UIDynamicPopup udp = CreatePopup(_selectAsset, true);
                udp.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);
                udp.popup.onValueChangeUnityEvent.AddListener(delegate { ChangeAsset(); });
                _selectAsset.choices = _assetList;
                _selectAsset.val = "None";

                var refreshListButton = CreateButton("Refresh Assetbundle List", true);
                refreshListButton.button.onClick.AddListener(RefreshiList);
                JSONStorableAction RefreshListAct = new JSONStorableAction("Refresh Assetbundle List", RefreshiList);

                _cuaScale = new JSONStorableFloat("CUA Scale", 1, 0.1f, 10f);
                var scaleSlider = CreateSlider(_cuaScale, true);
                scaleSlider.slider.onValueChanged.AddListener(delegate { ChangeScale(); });

                var atoms = SuperController.singleton.GetAtomUIDs();
                List<string> lights = new List<string>();
                lights.Add("None");
                foreach (var item in atoms)
                {
                    var atom = SuperController.singleton.GetAtomByUid(item);
                    if (atom.type == "InvisibleLight")
                    {
                        lights.Add(item);
                    }
                }

                _lightSelect = new JSONStorableStringChooser("Select Invisible Light", lights, "None", "Invisible Light");
                var lightSelectPop = CreatePopup(_lightSelect, true);
                lightSelectPop.popup.onValueChangeUnityEvent.AddListener(delegate { SelectLight(); });

                _intensity = new JSONStorableFloat("Invisible Light Intensity", 1f, 0f, 8f);
                var intensitySlider = CreateSlider(_intensity, true);
                intensitySlider.slider.onValueChanged.AddListener(delegate { ChangeIntensity(); });

                RegisterBool(_isEnableVar);
                RegisterStringChooser(_lightSelect);
                RegisterFloat(_intensity);
                RegisterFloat(_cuaScale);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        public void SelectDir()
        {
            _curretCUANumber = 0;
            var shortcuts = FileManagerSecure.GetShortCutsForDirectory(_cuaPath, false, false, false, false);

            if (_isEnableVar.val)
            {
                SuperController.singleton.GetDirectoryPathDialog(LoadCUAList, _cuaPath, shortcuts, false);
            }
            else
            {
                SuperController.singleton.GetDirectoryPathDialog(LoadCUAList, _cuaPath, null, false);
            }
        }

        public void LoadCUAList(string path)
        {
            if (path != null && path != "")
            {
                _isWaitRefreshList = false;
                _selectedPath = path;
                string[] cuaFiles;
                ArrayList allFiles = new ArrayList();
                GetAllFiles(path, "*." + _cuaExt, ref allFiles);
                cuaFiles = (string[])allFiles.ToArray(typeof(string));

                _cuaList = new List<string>();
                _cuaListText = "";

                foreach (var item in cuaFiles)
                {
                    var itemSplit = item.Replace("\\", "/").Replace("//", "/").Split('/');
                    var assetbundle = itemSplit[itemSplit.Length - 1];
                    var jpgName = item.Remove(item.LastIndexOf(_cuaExt)) + "jpg";
                    var jpgName2 = jpgName.Replace("\\", "/").Replace("//", "/");
                    if (jpgName2.Contains(":"))
                    {
                        jpgName2 = jpgName2.Replace(".var:", ":");
                        var fileSplit = jpgName2.Split(':');
                        var varSplit = fileSplit[0].Split('/');
                        var varName = varSplit[varSplit.Length - 1];
                        jpgName2 = _pluginDataPath + varName + fileSplit[1];
                    }

                    if (!FileManagerSecure.FileExists(jpgName) && !FileManagerSecure.FileExists(jpgName2) && assetbundle != "colorcorrect.assetbundle")
                    {
                        var tempCUA = item.Replace("\\", "/").Replace("//", "/");
                        _cuaList.Add(tempCUA);

                        if (_cuaListText.Length < _jsonstringLimit)
                        {
                            _cuaListText = _cuaListText + tempCUA.Replace(_cuaPath, "") + "\n";
                        }
                    }
                }

                _maxCUANumber = _cuaList.Count;

                if (_maxCUANumber < 1)
                {
                    _cuaListTF.val = "";
                    _curretCUANumber = 0;
                    return;
                }
                else
                {
                    _cuaListTF.val = _cuaListText;
                }

                if (_curretCUANumber >= _maxCUANumber)
                {
                    _curretCUANumber = _maxCUANumber - 1;
                }

                LoadCUA();
            }
        }

        public void NextCUA()
        {
            if (_maxCUANumber != 0)
            {
                _curretCUANumber++;
                if (_curretCUANumber >= _maxCUANumber)
                {
                    _curretCUANumber = 0;
                }

                LoadCUA();
            }
        }

        public void NextCUA2()
        {
            if (_maxCUANumber != 0)
            {
                _curretCUANumber += 10;
                if (_curretCUANumber >= _maxCUANumber)
                {
                    _curretCUANumber = _maxCUANumber;
                }

                LoadCUA();
            }
        }

        public void PreviousCUA()
        {
            if (_maxCUANumber != 0)
            {
                _curretCUANumber--;
                if (_curretCUANumber < 0)
                {
                    _curretCUANumber = _maxCUANumber - 1;
                }

                LoadCUA();
            }
        }

        public void PreviousCUA2()
        {
            if (_maxCUANumber != 0)
            {
                _curretCUANumber -= 10;
                if (_curretCUANumber < 0)
                {
                    _curretCUANumber = 0;
                }

                LoadCUA();
            }
        }

        public void LoadCUA()
        {
            if (_cuaList != null)
            {
                _currentCUATF.val = _cuaList[_curretCUANumber];
            }
        }

        public void SelectCUA()
        {
            StartCoroutine(SetAssetUrl());
        }

        IEnumerator SetAssetUrl()
        {
            _cuaControl.GetUrlJSONParam("assetUrl").val = _cuaList[_curretCUANumber];
            yield return new WaitForSeconds(1f);
            MakeAssetList();
        }

        public void MakeAssetList()
        {
            if (_cuaList != null)
            {
                var assetList = _cuaControl.GetStringChooserJSONParamChoices("assetName");
                _assetList = new List<string>();
                _assetListText = "";

                foreach (var item in assetList)
                {
                    _assetList.Add(item);
                    _assetListText = _assetListText + item + "\n";
                }
                _assetListTF.val = _assetListText;
                _selectAsset.choices = _assetList;

                if (_assetList != null && _assetList.Count > 1)
                {
                    var selectAsset = assetList[1];
                    _selectAsset.val = _assetList[1];
                    _cuaControl.GetStringChooserJSONParam("assetName").val = selectAsset;
                }

                _startTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;

                if (_assetList.Count > 1)
                {
                    _isNeedAssetRefresh = false;
                }
                else
                {
                    _isNeedAssetRefresh = true;
                }
            }
        }

        public void ChangeAsset()
        {
            _cuaControl.GetStringChooserJSONParam("assetName").val = _selectAsset.val;
        }

        public void SelectLight()
        {
            if (_lightSelect.val != null && _lightSelect.val != "None")
            {
                _invisibleLight = SuperController.singleton.GetAtomByUid(_lightSelect.val);
                _intensity.val = _invisibleLight.GetStorableByID("Light").GetFloatJSONParam("intensity").val;
            }
        }

        public void ChangeIntensity()
        {
            if (_invisibleLight != null)
            {
                _invisibleLight.GetStorableByID("Light").GetFloatJSONParam("intensity").val = _intensity.val;
            }
        }

        public void Screenshot()
        {
            var assetbundle = _currentCUATF.val;
            var jpgName = assetbundle.Remove(assetbundle.LastIndexOf(_cuaExt)) + "jpg";

            if (jpgName.Contains(":"))
            {
                jpgName = jpgName.Replace(".var:", ":");
                var fileSplit = jpgName.Split(':');
                var varSplit = fileSplit[0].Split('/');
                var varName = varSplit[varSplit.Length - 1];
                jpgName = _pluginDataPath + varName + fileSplit[1];
                var assetPath = jpgName.Remove(jpgName.LastIndexOf("/"));
                FileManagerSecure.CreateDirectory(assetPath);
            }

            if (FileManagerSecure.FileExists(jpgName))
            {
                return;
            }
            else
            {
                SuperController.singleton.DoSaveScreenshot(jpgName);
                _startTime2 = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                _isWaitRefreshList = true;
            }
        }

        public void RefreshiList()
        {
            LoadCUAList(_selectedPath);
        }

        private void ChangeScale()
        {
            _targetAtom.GetStorableByID("scale").GetFloatJSONParam("scale").val = _cuaScale.val;
        }

        public static void GetAllFiles(string folder, string filter, ref ArrayList allFiles)
        {
            string[] filses = FileManagerSecure.GetFiles(folder, filter);
            allFiles.AddRange(filses);

            string[] subFolders = FileManagerSecure.GetDirectories(folder);
            foreach (string subF in subFolders)
            {
                GetAllFiles(subF, filter, ref allFiles);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                if (_isNeedAssetRefresh)
                {
                    var currentTime = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    if (currentTime < _startTime)
                    {
                        currentTime += 60000;
                    }

                    if (currentTime - _startTime > 1000)
                    {
                        var assetList = _cuaControl.GetStringChooserJSONParamChoices("assetName");
                        if (assetList.Count > 1)
                        {
                            MakeAssetList();
                        }

                        _startTime = currentTime;
                    }
                }

                if (_isWaitRefreshList)
                {
                    var currentTime2 = DateTime.Now.Second * 1000 + DateTime.Now.Millisecond;
                    if (currentTime2 < _startTime2)
                    {
                        currentTime2 += 60000;
                    }

                    if (currentTime2 - _startTime2 > 8000)
                    {
                        RefreshiList();
                    }
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {
        }

    }
}